@@ -135,7 +135,7 @@ gst_cdaudio_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_metadata (element_class, "CD player",
+  gst_element_class_set_static_metadata (element_class, "CD player",
       "Generic/Bin",
       "Play CD audio through the CD Drive", "Wim Taymans <wim@fluendo.com>");
 
