@@ -96,7 +96,7 @@ gst_jasper_dec_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_jasper_dec_src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_jasper_dec_sink_template));
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "Jasper JPEG2000 image decoder", "Codec/Decoder/Image",
       "Decodes JPEG2000 encoded images using jasper",
       "Mark Nauwelaerts <mnauw@users.sf.net>");
