@@ -126,7 +126,7 @@ gst_mim_enc_class_init (GstMimEncClass * klass)
       gst_static_pad_template_get (&src_factory));
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&sink_factory));
-  gst_element_class_set_metadata (gstelement_class,
+  gst_element_class_set_static_metadata (gstelement_class,
       "Mimic Encoder",
       "Codec/Encoder/Video",
       "MSN Messenger compatible Mimic video encoder element",
