@@ -158,7 +158,7 @@ gst_edge_detect_class_init (GstEdgeDetectClass * klass)
           "Aperture size for Sobel operator (Must be either 3, 5 or 7", 3, 7, 3,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "edgedetect",
       "Filter/Effect/Video",
       "Performs canny edge detection on videos and images.",
