@@ -92,7 +92,7 @@ gst_rgb2bayer_class_init (GstRGB2BayerClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rgb2bayer_sink_template));
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "RGB to Bayer converter",
       "Filter/Converter/Video",
       "Converts video/x-raw to video/x-bayer",
