@@ -176,7 +176,7 @@ gst_compare_class_init (GstCompareClass * klass)
       gst_static_pad_template_get (&sink_factory));
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&check_sink_factory));
-  gst_element_class_set_metadata (gstelement_class, "Compare buffers",
+  gst_element_class_set_static_metadata (gstelement_class, "Compare buffers",
       "Filter/Debug", "Compares incoming buffers",
       "Mark Nauwelaerts <mark.nauwelaerts@collabora.co.uk>");
 }
