@@ -116,7 +116,7 @@ gst_fisheye_class_init (GstFisheyeClass * klass)
   gstgt_class = (GstGeometricTransformClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
-  gst_element_class_set_metadata (gstelement_class,
+  gst_element_class_set_static_metadata (gstelement_class,
       "fisheye",
       "Transform/Effect/Video",
       "Split the image into two halves and reflect one over each other",
