@@ -105,7 +105,7 @@ gst_inter_audio_src_class_init (GstInterAudioSrcClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_inter_audio_src_src_template));
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "Internal audio source",
       "Source/Audio",
       "Virtual audio source for internal process communication",
