@@ -111,7 +111,7 @@ gst_y4m_dec_class_init (GstY4mDecClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_y4m_dec_sink_template));
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "YUV4MPEG demuxer/decoder", "Codec/Demuxer",
       "Demuxes/decodes YUV4MPEG streams", "David Schleef <ds@schleef.org>");
 }
