@@ -562,7 +562,7 @@ gst_avf_video_src_base_init (gpointer gclass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
-  gst_element_class_set_details_simple (element_class,
+  gst_element_class_set_metadata (element_class,
       "Video Source (AVFoundation)", "Source/Video",
       "Reads frames from an iOS AVFoundation device",
       "Ole André Vadla Ravnås <oravnas@cisco.com>");
